package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.FechamentoParamgeral;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;

@Controller("busquedaFechamentoParamgeralController")
@Scope("prototype")
public class BusquedaFechamentoParamgeralController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	
	@Autowired
    private transient PagedListWrapper<FechamentoParamgeral> plwParams;
	private Paging pagingParams;
	
	private MyListbox paramsList;
	
	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresas;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {		
		
		lsEmpresas = UsuarioLogado.getUsuarioLogado().getEmpresa();
		
		super.doAfterCompose(comp);
		
		paramsList.setItemRenderer(new FechamentoParamgeralListItemRenderer());
		paramsList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				FechamentoParamgeral params = (FechamentoParamgeral) paramsList.getSelected();
				visualizaParams(params);
			}
		});
		
		actualizaParamsList();
	}

	
	private void visualizaParams(FechamentoParamgeral _params){
		
		if (_params == null)
			return;
		
		Map<String,Object> args = new HashMap<String,Object>();
		args.put("fechamentoparamgeralId", _params.getFechamentoparamgeralId());
		args.put("paramsList", paramsList);

		openWindow("/gui/configuraciones_comerciales/editarFechamentoParamgeral.zul",
				Labels.getLabel("editarFechamentoParamgeralController.window.title"), args, MODAL);
		
	}
	
	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		actualizaParamsList();
	}

	public void onClick$btnRefresh(Event ev) {
		actualizaParamsList();
	}

	public void onClick$btnNovo(Event ev) {
		FechamentoParamgeral fechamentoParamgeral = new FechamentoParamgeral();
		fechamentoParamgeral.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		fechamentoParamgeral.setActivo(Boolean.TRUE);
		visualizaParams(fechamentoParamgeral);
	}
	
	private void actualizaParamsList() {
		HibernateSearchObject<FechamentoParamgeral> paramsBusqueda =
				new HibernateSearchObject<FechamentoParamgeral>(FechamentoParamgeral.class, pagingParams.getPageSize());

		paramsBusqueda.addFilterEqual("activo", true);
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();		
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			paramsBusqueda.addFilterEqual("empresa", empresa);
		} else {
			paramsBusqueda.addFilterIn("empresa", lsEmpresas);
		}
		
		paramsBusqueda.addSortDesc("fechamentoparamgeralId");

		plwParams.init(paramsBusqueda, paramsList, pagingParams);

		if (paramsList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaFechamentoParamgeralController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}


	public MyListbox getParamsList() {
		return paramsList;
	}


	public void setParamsList(MyListbox paramsList) {
		this.paramsList = paramsList;
	}


	public Combobox getCmbEmpresa() {
		return cmbEmpresa;
	}


	public void setCmbEmpresa(Combobox cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}


	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}


	public void setLsEmpresas(List<Empresa> lsEmpresas) {
		this.lsEmpresas = lsEmpresas;
	}
}
