package com.rjconsultores.ventaboletos.web.gui.controladores.tarifas;

import java.util.HashMap;
import java.util.Map;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.PrecoApanhe;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderPrecoApanhe;

@Scope("prototype")
@Controller("busquedaPrecoApanheController")
public class BusquedaPrecoApanheController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	
	private static Logger log = LogManager.getLogger(BusquedaPrecoApanheController.class);

	@Autowired
    private transient PagedListWrapper<PrecoApanhe> plwPrecoApanhe;
    private MyListbox precoApanheList;
    private Paging pagingPrecoApanhe;

    @Override
    public void doAfterCompose(Component comp) throws Exception {
        super.doAfterCompose(comp);

        precoApanheList.setItemRenderer(new RenderPrecoApanhe());
        precoApanheList.addEventListener("onDoubleClick", new EventListener() {

            @Override
            public void onEvent(Event event) throws Exception {
            	PrecoApanhe a = (PrecoApanhe) precoApanheList.getSelected();
                verPrecoApanhe(a);
            }
        });

        refreshLista();

    }

    private void verPrecoApanhe(PrecoApanhe t) {
        if (t == null) {
            return;
        }

        Map<String, Object> args = new HashMap<String, Object>();
        args.put("precoApanhe", t);
        args.put("precoApanheList", precoApanheList);

        openWindow("/gui/tarifas/editarPrecoApanhe.zul",
                Labels.getLabel("editarPrecoApanheController.window.title"), args, MODAL);
    }

    private void refreshLista() {
        HibernateSearchObject<PrecoApanhe> precoApanhes = new HibernateSearchObject<PrecoApanhe>(PrecoApanhe.class, pagingPrecoApanhe.getPageSize());

        precoApanhes.addFilterEqual("activo", Boolean.TRUE);
        plwPrecoApanhe.init(precoApanhes, precoApanheList, pagingPrecoApanhe);

        if (precoApanheList.getData().length == 0) {
            try {
                Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
                        		Labels.getLabel("busquedaPrecoApanheController.window.title"),
                        		Messagebox.OK, Messagebox.INFORMATION);
            } catch (InterruptedException e) {
            	log.error(e.getMessage(), e.getCause());
            }
        }
    }

    public void onClick$btnPesquisa(Event ev) {
        refreshLista();
    }

    public void onClick$btnRefresh(Event ev) {
        refreshLista();
    }

    public void onClick$btnNovo(Event ev) {
        verPrecoApanhe(new PrecoApanhe());
    }

	public MyListbox getPrecoApanheList() {
		return precoApanheList;
	}

	public void setPrecoApanheList(MyListbox precoApanheList) {
		this.precoApanheList = precoApanheList;
	}

	public Paging getPagingPrecoApanhe() {
		return pagingPrecoApanhe;
	}

	public void setPagingPrecoApanhe(Paging pagingPrecoApanhe) {
		this.pagingPrecoApanhe = pagingPrecoApanhe;
	}
}
